/*
 * Jump N Run game Demo
 *
 * A Plan Adr = c000
 * window adr = b000
 * B Plan adr = e000
 * sprite adr = be00
 * HScrol adr = b800
 *
 * Rev1.2       23Sep99
 */

#include "genesis.h"
#include "tile.h"
#include "pal.h"
#include "nuage.h"


/** address of Mick's tiles **/
/* since Mick something like 
*               ****
*               ****
*               ****
*               ****
*               ****
* when each * = 1 tile = 32bytes
*/
#define ADR_MICK0       ( 32 + (32 * 4*11) + (32 *5 * 2) + (32 * 5 * 2) )
#define ADR_MICK1       ( ADR_MICK0 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_MICK2       ( ADR_MICK1 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_MICK3       ( ADR_MICK2 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_MICK4       ( ADR_MICK3 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_MICK5       ( ADR_MICK4 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_NUAGE1      ( ADR_MICK5 + (32 * 4 * 4) + (32 * 4 *1) )
#define ADR_NUAGE2      ( ADR_NUAGE1 + (32 * 32 * 14) )

/** to know where Mick goes **/
#define LEFT            1
#define RIGHT           0

#define STANDUP         0
#define WALK            1
#define RUN             2

/***** MODIFY THE WIDTH ****/
/***** see INIT_GFX     ****/

uint  extWIDTH;


/************** Function prototypes *************/
void init_gfx();
void Redraw();
void Anim_Mick();
void Jump_Mick(); 
uint IsContact();

/*************  External Vars *************/

/**** in TILE.S ***********/
/**** tiles for Map *******/
extern ulong tile1[];
extern ulong tile2[];
extern ulong tile3[];

/**** in MAP_DEF.S ********/
/**** Def of the Map ******/
/* What tile at what pos? */
extern uint map_def[200*28];

/**** in MICK.S ***********/
/**** tiles for Mick ******/
extern uint mick0_a[];
extern uint mick0_b[];
extern uint mick1_a[];
extern uint mick1_b[];
extern uint mick2_a[];
extern uint mick2_b[];
extern uint mick3_a[];
extern uint mick3_b[];
extern uint mick4_a[];
extern uint mick4_b[];
extern uint mick5_a[];
extern uint mick5_b[];

/**** tiles for Nuage ******/
extern uint nuage1[];
extern uint nuage2[];



/*************    Variables  *************/
uint  step;             /* actual step of Mick's walk   */
uint  posAct;           /* actual relative pos of Mick  */
uint  Dir, lastDir;     /* where Mick goes, went        */
int xwrite1, xwrite2;   /* see Redraw() & scroll.txt    */

struct
{
        uint posx;
        uint posy;
        uint speed;
}Mick;


/*************  Main program  ***********/
void main()
{
    register uint x, y, xmod, joy;
    register uint tmp;
    register uint *pw;
    register ulong *pl;
    
    /* Initialize Genesis GFX registers */
    init_gfx();

    /* Wait GFX reg are initialized */
    wait_sync();
    
    /* Load the tile for map */
    dma_vram_copy(tile1, 0, 32 + 32 * 4*11 );
    dma_vram_copy(tile2, 32 + 32 * 4*11 , 32 * 5 * 2);
    dma_vram_copy(tile3, 32 + 32 * 4*11 + 32 *5 * 2, 32 * 5 * 2);
         /*              ^^^^^^^... this define ADR_MICK0, adjust ADR_MICK0 if you change this */

    /* Load Mick Steps */
    dma_vram_copy(mick0_a, ADR_MICK0, 32 * 4 * 4);
    dma_vram_copy(mick0_b, ADR_MICK0+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(mick1_a, ADR_MICK1, 32 * 4 * 4);
    dma_vram_copy(mick1_b, ADR_MICK1+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(mick2_a, ADR_MICK2, 32 * 4 * 4);
    dma_vram_copy(mick2_b, ADR_MICK2+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(mick3_a, ADR_MICK3, 32 * 4 * 4);
    dma_vram_copy(mick3_b, ADR_MICK3+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(mick4_a, ADR_MICK4, 32 * 4 * 4);
    dma_vram_copy(mick4_b, ADR_MICK4+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(mick5_a, ADR_MICK5, 32 * 4 * 4);
    dma_vram_copy(mick5_b, ADR_MICK5+ 32 * 4 * 4, 32 * 4 *1);
    dma_vram_copy(nuage1,  ADR_NUAGE1, 32 * 40 * 15);
    dma_vram_copy(nuage2,  ADR_NUAGE2, 32 * 40 * 15);



    /* Draw the first 48 columns of the Map */
    for (y=0; y< 28; y++)
        for (x=0;x < 48; x++)
            show_tiles(map_def[x+(y*200)], map_def[x+(y*200)], 0, x, y, APLAN, 0, 0, 0);
   

    /* Nuage (cloud) is made of 4 blocks of 14*16tiles (because of JAS memory, i had to split it) */
    for (y=0; y<30; y++)
    {
        show_tiles(16*y +  ADR_NUAGE1/32, 16 + 16*y + ADR_NUAGE1/32, 2, 0, y, BPLAN, 0, 0, 0);
        show_tiles(16*y +  ADR_NUAGE2/32, 16 + 16*y + ADR_NUAGE2/32, 2, 16, y, BPLAN, 0, 0, 0);
        show_tiles(16*y +  ADR_NUAGE1/32, 16 + 16*y + ADR_NUAGE1/32, 2, 32, y, BPLAN, 0, 0, 0);
        show_tiles(16*y +  ADR_NUAGE2/32, 16 + 16*y + ADR_NUAGE2/32, 2, 48, y, BPLAN, 0, 0, 0);
    }
    wait_sync();

    /* Define the palettes (now we can see the fields) */
    set_colors(0, pal_tile);
    set_colors(1, pal_mick);
    set_colors(2, pal_nuage);

    /* Mick is in pos 1, standing up and ready to go right */
    posAct = 0;
    step = 0;
    Dir = RIGHT;
    lastDir = RIGHT;
    
    
    Mick.speed = WALK;    

    /* Pos 1 = 8pixels of the LeftScreen */
    Mick.posx=8;
    
    /* Test if his feet touch the map, 2 tiles under the head */
    for (y=0; y< 28; y++)
        if (map_def[(y*200) +2+ (posAct/8)] == 0)  Mick.posy=( y*8 - (2*8));
                                                 /* adjust to his head^^^, remember we test the feet! */
        
    def_sprite(1,Mick.posx,Mick.posy,0xf00 ,0xA000 + (ADR_MICK0/32));
    def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK0/32));
    show_sprite(1,2);

    xwrite1 = 48*8;
    xwrite2 = 49*8;
    
    init_joypad();
    
    /* This look stupid but it seems the first call to Read_joypad return a bad value */
    joy= read_joypad1();

    while(1)
    {
        joy= read_joypad1();

        if (joy)
        {
                switch(joy & 0xff)
                {
                        case JOY_RIGHT:
                                Mick.speed = WALK;
                                if (posAct < (200*8 - 32*8 -8) )
                                {
                                        posAct++;
                                        step++;
                                        xwrite1++;
                                        xwrite2++;
                                        lastDir = Dir;
                                        Dir = RIGHT;
                                }
                                else
                                        step = 0;
                                break;
                        case JOY_LEFT:
                                Mick.speed = WALK;
                                if (posAct > 0)
                                {
                                        posAct--;
                                        xwrite1--;
                                        xwrite2--;
                                        step++;
                                        lastDir = Dir;
                                        Dir = LEFT;
                                }
                                else
                                        step = 0;
                                break;
                        case BUTTON_A:                  /* Jump */
                                Mick.speed = STANDUP;
                                lastDir = Dir;
                                step= 1;
                                Jump_Mick();
                                step= 0;
                                Anim_Mick();
                                break;
                        case JOY_RIGHT + BUTTON_A:      /* Jump to the right */
                                Mick.speed = WALK;
                                lastDir = Dir;
                                Dir = RIGHT;
                                step= 1;
                                Jump_Mick();
                                step= 0;
                                Dir = lastDir;
                                break;
                        case JOY_LEFT + BUTTON_A:       /* Jump to the left */
                                Mick.speed = WALK;
                                lastDir = Dir;
                                Dir = LEFT;
                                step= 1;
                                Jump_Mick();
                                step= 0;
                                Dir = lastDir;
                                break;
                        case JOY_RIGHT + BUTTON_B:
                                Mick.speed = RUN;
                                if (posAct < (200*8 - 32*8 -8 -1) )
                                {
                                        posAct+=2;
                                        step++;
                                        xwrite1+=2;
                                        xwrite2+=2;
                                        lastDir = Dir;
                                        Dir = RIGHT;
                                }
                                else
                                        step = 0;
                                break;
                        case JOY_LEFT + BUTTON_B:
                                Mick.speed = RUN;
                                if (posAct > 1)
                                {
                                        posAct-=2;
                                        xwrite1-=2;
                                        xwrite2-=2;
                                        step++;
                                        lastDir = Dir;
                                        Dir = LEFT;
                                }
                                else
                                        step = 0;
                                break;
                        case JOY_RIGHT + BUTTON_B + BUTTON_A:      /* Jump running to the right */
                                Mick.speed = RUN;
                                lastDir = Dir;
                                Dir = RIGHT;
                                step= 1;
                                Jump_Mick();
                                step= 0;
                                Dir = lastDir;
                                break;
                        case JOY_LEFT + BUTTON_B + BUTTON_A:       /* Jump running to the left */
                                Mick.speed = RUN;
                                lastDir = Dir;
                                Dir = LEFT;
                                step= 1;
                                Jump_Mick();
                                step= 0;
                                Dir = lastDir;
                                break;
                        default:
                                step=0;
                }

                for (y=0; y< 28; y++)
                        if (map_def[(y*200) +2+ (posAct/8)] == 0)  Mick.posy=( y*8 - (2*8)); 
                                          /* ^ define by Mick.posx                    ^^^adjust to his feet */
                                
                Redraw();
        }
    }
};

/*************  Initialize GFX  ***********/
void init_gfx()
{
    register uint *pw;

    pw = (uint *) GFXCNTL;

    *pw = 0x8016;   /* reg. 0 - Enable HBL */
    *pw = 0x8174;   /* reg. 1 - Enable display, VBL, DMA + VCell size */
    *pw = 0x8230;   /* reg. 2 - Plane A =$30*$400=$C000 */
    *pw = 0x832C;   /* reg. 3 - Window  =$2C*$400=$B000 */
    *pw = 0x8407;   /* reg. 4 - Plane B =$7*$2000=$E000 */
    *pw = 0x855F;   /* reg. 5 - sprite table begins at $BE00=$5F*$200 */
    *pw = 0x8600;   /* reg. 6 - not used */
    *pw = 0x8702;   /* reg. 7 - Background Color number*/
    *pw = 0x8800;   /* reg. 8 - not used */
    *pw = 0x8900;   /* reg. 9 - not used */
    *pw = 0x8a01;   /* reg 10 - HInterrupt timing */
    *pw = 0x8b00;   /* reg 11 - $0000abcd a=extr.int b=vscr cd=hscr */
    *pw = 0x8c08;   /* reg 12 - hcell mode + shadow/highight + interlaced mode */
    *pw = 0x8d2E;   /* reg 13 - HScoll Table = $B800 */
    *pw = 0x8e00;   /* reg 14 - not used */
    *pw = 0x8f02;   /* reg 15 - auto increment data */
    *pw = 0x9001;   /* reg 16 - scrl screen v&h size */
    
    extWIDTH = 64;
    
    *pw = 0x9100;   /* reg 17 - window hpos */
    *pw = 0x92ff;   /* reg 18 - window vpos */
};


/*************  Redraw sub-program  ***********/
void Redraw()
{
        register uint x, y, xmod;
        
        x = posAct/8;
        
        if (xwrite1<=-1)    xwrite1 = 7+ (63*8);
        if (xwrite1/8>=64)  xwrite1 = 0;
        if (xwrite2<=-1)    xwrite2 = 7+ (63*8);
        if (xwrite2/8>=64)  xwrite2 = 0;

        Anim_Mick(); 
        scrollh(-posAct, -(posAct/3) );
        
        for (y=0; y< 28; y++)
                show_tiles(map_def[x+48+(y*200)], map_def[x+48+(y*200)], 0, xwrite1/8, y, APLAN, 0, 0, 0);

        for (y=0; y< 28; y++) 
                show_tiles(map_def[x-15+(y*200)], map_def[x-15+(y*200)], 0, xwrite2/8, y, APLAN, 0, 0, 0);
}


/*************  Anim_Mick sub-program  ***********/
void Anim_Mick()
{
        switch (step)
        {
                /* Dir << 11  to invert Mick if he goes left, see DEF_SPRITE() in GENESIS.C for more details */
                case 0:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00 ,0xA000 + (ADR_MICK0/32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK0/32) + (Dir << 11));
                        break;
                case 1:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00,0xA000 + (ADR_MICK1 / 32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK1 / 32) + (Dir << 11) );                        
                        break;
                case 4:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00,0xA000 + (ADR_MICK2 / 32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK2 / 32) + (Dir << 11) );
                        break;
                case 7:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00,0xA000 + (ADR_MICK3 / 32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK3 / 32) + (Dir << 11) );
                        break;
                case 10:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00,0xA000 + (ADR_MICK4 / 32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK4 / 32) + (Dir << 11) );
                        break;
                case 13:
                        def_sprite(1,Mick.posx,Mick.posy,0xf00,0xA000 + (ADR_MICK5 / 32) + (Dir << 11) );
                        def_sprite(2,Mick.posx,Mick.posy+31,0xc00,0xA000 + (16 + ADR_MICK5 / 32) + (Dir << 11) );
                        break;
                case 15:
                        step=1;

        }
        show_sprite(1,2);
};


/*************  Jump_Mick sub-program  ***********/
void Jump_Mick()
{
        register uint y, x, Upto ;
        
        x=1;

        for (y=0; y<40 & x ; y++)
        {
                /* if (map_def[((Mick.posy)*200) +(posAct/8)] != 0)  y=100; */
                /* Head touchs something? then Stop */
                
                

                Mick.posy--;
                scrollv(-(y/5), 0); 
                if (Dir == RIGHT)
                {
                        if (posAct < (200*8 - 32*8 -8) )
                        {
                                posAct+=Mick.speed;
                                xwrite1+=Mick.speed;
                                xwrite2+=Mick.speed;
                        }
                }
                else
                {
                        if (posAct>0)
                        {
                                posAct-=Mick.speed;
                                xwrite1-=Mick.speed;
                                xwrite2-=Mick.speed;
                        }
                 }
                 Redraw();
                 if ( IsContact() )  x=0;                
        }

        x = 1;

        while (x)
        {

                /* if (map_def[ ((Mick.posy/8)*200) + 3*200 + (posAct/8) + 4 ] != 0)  x=0; 
                /* Right Feet inside 2 tiles of grasse? Then Stop (3 = Mick Height else we'll take his head as ref!!) */

                Mick.posy++;

                /*scroll down to initial EVEN if he goes more down */
                scrollv(-(y/5), 0);
                if (y>0) y--;
                       
                if (Dir == RIGHT)
                {
                        if (posAct < (200*8 - 32*8 -8) )
                        {
                                posAct+= Mick.speed;
                                xwrite1+= Mick.speed;
                                xwrite2+= Mick.speed;
                        }
                }
                else
                {
                        if (posAct > 0)
                        {
                                posAct-= Mick.speed;
                                xwrite1-= Mick.speed;
                                xwrite2-= Mick.speed;
                        }
                }
                Redraw();

                if (Mick.posy>0 & IsContact() )  
                        {
                                x=0;
                                scrollv(0, 0); 
                                /* Hard way because the difference is not enought to see a blink */
                                /* See Doc for more details */
                        }
                 
        }
};


/*************  IsContact sub-program  ***********/
uint IsContact()
{
     
        if (map_def[ ((Mick.posy/8)*200) + 2 + 3*200 + (posAct/8) ] != 0)
                return 1;
        else
                return 0;
}

